#!/bin/bash

#	Shell скрипт для обновления системного программного обеспечения в дистрибутиве Linux базовой станции LoRaWAN
#	У скрпта 2 режима работы.
#
#	1. Основной (без параметров, по-умолчанию). Проверяет систему и, при необходимости, обновляет. Запускать только от root.
#	   > sudo ./bs_lora_update_script.sh
#
#	2. Только проверить необходимость обновления дистрибутива, но при этом само обновление не проводить (ключ --check).
#	   > ./bs_lora_update_script.sh --check


# Ключ "--help": краткая справка по использованию скрипта
if [[ "$1" == --help ]]; then
	echo "Usage:"
	echo -e "\tWithout parameters\tUpdate distribution componentes if needed (root only)"
	echo -e "\t--check\t\t\tOnly check for update is needed"
	echo -e "\t--help\t\t\tThis help message"
	exit 0
fi

# Ключ "--check": только проверка необходимости обновления
TEXT_UPDATE_YES='FIXED'
TEXT_UPDATE_NO='FIX NOT NEEDED'
if [[ "$1" == --check ]]; then
	echo "Checking for update is needed:"
	TEXT_UPDATE_YES='NEED FIX'
	FLAG_CHECKONLY=1
fi

# Проверка на суперпользователя; скрипт нужно запускать от root (uid = 0)
if [ "$(id -u)" -ne 0 ]; then
	if [[ ${FLAG_CHECKONLY} != 1 ]]; then
		echo "Please run as root:"
		echo -e "\t> sudo ./bs_lora_update_script.sh"
		exit 1
	fi
fi

# Переменные для раскрашивания статусов в разные цвета
COLOR_RED='\033[0;31m'
COLOR_YELLOW='\033[1;33m'
COLOR_GREEN='\033[0;32m'
COLOR_NO='\033[0m'

# Подавление сообщений из stderr
exec 2>/dev/null

# Если tun.ko нет ни в виде модуля, вкомпилированного в ядро, ни в виде внешнего загружаемого модуля, то добавить tun.ko как внешний модуль
printf "VPN Client lack of tun problem\t\t\t"
if ! grep -q "kernel/drivers/net/tun.ko" /lib/modules/`uname -r`/modules.builtin && ! [ -f /lib/modules/`uname -r`/kernel/drivers/net/tun.ko ]; then
	if [[ ${FLAG_CHECKONLY} != 1 ]]; then
		cp update_files/tun.ko /lib/modules/`uname -r`/kernel/drivers/net/
		depmod
		echo "tun" > /etc/modules-load.d/tun.conf
		insmod /lib/modules/`uname -r`/kernel/drivers/net/tun.ko >/dev/null
	fi
	echo -e [${COLOR_YELLOW}${TEXT_UPDATE_YES}${COLOR_NO}]
else
	echo -e [${COLOR_GREEN}${TEXT_UPDATE_NO}${COLOR_NO}]
fi

# Если не установлен пакет, содержащий net-snmp-create-v3-user, то установить его
printf "SNMP lack of some tools\t\t\t\t"
if ! [ -f /usr/bin/net-snmp-create-v3-user ]; then
	if [[ ${FLAG_CHECKONLY} != 1 ]]; then
		dpkg -i update_files/net-snmp-dev_5.8-r0_armhf.deb >/dev/null
	fi
	echo -e [${COLOR_YELLOW}${TEXT_UPDATE_YES}${COLOR_NO}]
else
	echo -e [${COLOR_GREEN}${TEXT_UPDATE_NO}${COLOR_NO}]
fi

# Если пользователь snmpadmin для сервера SNMP v3 не создан, то создать его
printf "SNMP v3 lack of created user\t\t\t"
if ! grep -q "rouser snmpadmin" /usr/share/snmp/snmpd.conf; then
	if [[ ${FLAG_CHECKONLY} != 1 ]]; then
		systemctl stop snmpd
		net-snmp-create-v3-user -ro -A str_passwd -a SHA -X passwd22 -x AES snmpadmin >/dev/null
		systemctl start snmpd
	fi
	echo -e [${COLOR_YELLOW}${TEXT_UPDATE_YES}${COLOR_NO}]
else
	echo -e [${COLOR_GREEN}${TEXT_UPDATE_NO}${COLOR_NO}]
fi
